/* ScummVM - Graphic Adventure Engine
 *
 * ScummVM is the legal property of its developers, whose names
 * are too numerous to list here. Please refer to the COPYRIGHT
 * file distributed with this source distribution.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#define FORBIDDEN_SYMBOL_EXCEPTION_exit

#if __APPLE__
    #include "TargetConditionals.h"
#endif

#include "common/textconsole.h"
#include "common/system.h"
#include "common/str.h"

namespace Common {

static OutputFormatter s_errorOutputFormatter = 0;

void setErrorOutputFormatter(OutputFormatter f) {
	s_errorOutputFormatter = f;
}

static ErrorHandler s_errorHandler = 0;

void setErrorHandler(ErrorHandler handler) {
	s_errorHandler = handler;
}


} // End of namespace Common


#ifndef DISABLE_TEXT_CONSOLE

void warning(const char *s, ...) {
	Common::String output;
	va_list va;

	va_start(va, s);
	output = Common::String::vformat(s, va);
	va_end(va);

	output = "WARNING: " + output + "!\n";

	if (g_system)
		g_system->logMessage(LogMessageType::kWarning, output.c_str());
	// TODO: Think of a good fallback in case we do not have
	// any OSystem yet.
}


void debugLog(const char *s, ...) {
    
    Common::String output;

    // HACK
#if (TARGET_OS_IPHONE)
    
    static char buf[65536];
    va_list va;
    va_start(va, s);
    vsprintf(buf, s, va);

     va_end(va);
    
    output = buf;
    
#else
    
    va_list va;
    va_start(va, s);
    output = Common::String::vformat(s, va);

	// TEMP: add milliseconds to debug logs
	//output = Common::String::format("%u: %s", g_system->getMillis(), output.c_str());
    va_end(va);
    
    
#endif
    
    output = output + "\n";


    
    if (g_system)
        g_system->logMessage(LogMessageType::kDebug, output.c_str());
    // TODO: Think of a good fallback in case we do not have
    // any OSystem yet.
}

#endif

void error(const char *s, ...) {
    
    Common::String output;
    
    // HACK
#if (TARGET_OS_IPHONE)
    
    static char buf[65536];
    va_list va;
    va_start(va, s);
    vsprintf(buf, s, va);
    
    va_end(va);
    
    output = buf;
    
#else
    
    va_list va;
    va_start(va, s);
    output = Common::String::vformat(s, va);
    va_end(va);
    
    
#endif
    
    output = output + "\n";
    
    
    
    if (g_system)
        g_system->logMessage(LogMessageType::kError, output.c_str());

}
