/*
* The ScummVM WPF UI project
* Net Wrapper for WPF UI
*
* Copyright (c) 2016 MojoTouch
* Author: Ilya Rovensky
*/

#include "stdafx.h"
#include "UnmanagedWrapper.h"
#include "ManagedWrapper.h"
#include <vcclr.h>

using namespace System::Text;

namespace NetWrapper {

	String^ WCharToString(const wchar_t* input, bool upperCase)
	{
		String^ output = Encoding::UTF8->GetString(Encoding::GetEncoding(1252)->GetBytes(gcnew String(input)));

		return upperCase ? output->ToUpper() : output;
	}

	// Repacks a vector<wstring> to List<String>
	List<String^>^ VectorToList(vector<const wchar_t*>& vector, bool upperCase)
	{
		List<String^>^ result = gcnew List<String^>(vector.size());

		for (std::vector<const wchar_t*>::iterator iterator = vector.begin(); iterator != vector.end(); iterator++)
			result->Add(WCharToString(*iterator, upperCase));

		return result;
	}

	UnmanagedWrapper::UnmanagedWrapper()
	{
		_managedWrapper = new ManagedWrapper();
	}

	UnmanagedWrapper::~UnmanagedWrapper()
	{
		delete _managedWrapper;
	}

	int UnmanagedWrapper::ShowMenu(int menuAction, int menuState)
	{
		return (int)_managedWrapper->ShowMenu((MenuAction)menuAction, menuState);
	}

	void UnmanagedWrapper::DisplayMainMenu(int menuState)
	{
		_managedWrapper->DisplayMainMenu(menuState);
	}

	int UnmanagedWrapper::ShowSettings(int menuAction, vector<ConvertedSetting*>& settings)
	{
		// repack settings
		List<Setting^>^ settingList = gcnew List<Setting^>(settings.size());

		for (vector<ConvertedSetting*>::iterator iterator = settings.begin(); iterator != settings.end(); iterator++)
		{
			const ConvertedSetting* oldSetting = *iterator;
			Setting^ newSetting;

			switch (oldSetting->Type)
			{
			case SettingControlType::RADIO_GROUP:
				newSetting = gcnew WpfInterface::RadioSetting();			
				
				((RadioSetting^)newSetting)->ItemNames = VectorToList(*((ConvertedRadioSetting*)oldSetting)->ItemNames, UpperCase);
				((RadioSetting^)newSetting)->ItemDescriptions = VectorToList(*((ConvertedRadioSetting*)oldSetting)->ItemDescriptions, UpperCase);
				break;
			case SettingControlType::SLIDER:
				newSetting = gcnew WpfInterface::SliderSetting();

				((WpfInterface::SliderSetting^)newSetting)->MaxValue = ((ConvertedSliderSetting*)oldSetting)->MaxValue;
				((WpfInterface::SliderSetting^)newSetting)->MinValue = ((ConvertedSliderSetting*)oldSetting)->MinValue;
				((WpfInterface::SliderSetting^)newSetting)->Title = WCharToString(((ConvertedSliderSetting*)oldSetting)->Title, UpperCase);
				break;
			}
			
			newSetting->Value = oldSetting->Value;
					
			vector<bool> bools = oldSetting->States;
			newSetting->States = gcnew List<bool>(oldSetting->States.size());
			for (vector<bool>::iterator boolIterator = bools.begin(); boolIterator != bools.end(); boolIterator++)
				newSetting->States->Add((*boolIterator));

			settingList->Add(newSetting);
		}

		int result = _managedWrapper->ShowSettings((MenuAction)menuAction, settingList);

		// update values
		for (int i = 0; i < settingList->Count; i++)
			settings[i]->Value = settingList[i]->Value;

		return result;
	}

	void UnmanagedWrapper::HideMenu()
	{
		_managedWrapper->HideMenu();
	}

	/*void UnmanagedWrapper::ShowTutorial()
	{
		_managedWrapper->ShowTutorial();
	}*/

	int UnmanagedWrapper::ShowDialog(const wchar_t* message, vector<const wchar_t*>& buttons)
	{
		return _managedWrapper->ShowDialog(WCharToString(message, UpperCase), VectorToList(buttons, UpperCase));
	}

	int UnmanagedWrapper::ShowDialogWithList(const wchar_t* message, vector<const wchar_t*>& items, int buttonCount, const wchar_t* negativeButton, const wchar_t* positiveButton)
	{
		int result = _managedWrapper->ShowDialogWithList(WCharToString(message, UpperCase), VectorToList(items, UpperCase),
			buttonCount, WCharToString(negativeButton, UpperCase), WCharToString(positiveButton, UpperCase));
		
		return result;
	}

	int UnmanagedWrapper::ShowDialogWithInput(const wchar_t* message, wchar_t** input, const wchar_t* negativeButton, const wchar_t* positiveButton)
	{
		String^ managedInput = WCharToString(*input, UpperCase);

		int result = _managedWrapper->ShowDialogWithInput(WCharToString(message, UpperCase), managedInput,
			WCharToString(negativeButton, UpperCase), WCharToString(positiveButton, UpperCase));

		pin_ptr<const wchar_t> wch = PtrToStringChars(managedInput);
		*input = const_cast<wchar_t*>(wch);
		return result;
	}

	void UnmanagedWrapper::ShowProgressDialog(const wchar_t* message)
	{
		_managedWrapper->ShowProgressDialog(WCharToString(message, UpperCase));
	}

	void UnmanagedWrapper::HideProgressDialog()
	{
		_managedWrapper->HideProgressDialog();
	}

	void UnmanagedWrapper::ShowNotification(const wchar_t * message)
	{
		_managedWrapper->ShowNotification(WCharToString(message, UpperCase));
	}

	void UnmanagedWrapper::SetOglWindowHandle(HWND handle)
	{
		IntPtr managedHandle(handle);
		_managedWrapper->SetOglWindowHandle(managedHandle);
	}

	void UnmanagedWrapper::InitStrings(vector<const wchar_t*>& strings, int gameToPromote, const wchar_t* urlToPromote, const wchar_t* urlToRate)
	{
		_managedWrapper->InitStrings(VectorToList(strings, UpperCase), gameToPromote, WCharToString(urlToPromote, false), WCharToString(urlToRate, false));
	}

	bool UnmanagedWrapper::HasTouchScreen()
	{
		return _managedWrapper->HasTouchInput();
	}

	bool UnmanagedWrapper::IsShowingMenu()
	{
		return _managedWrapper->IsShowingMenu();
	}

	const wchar_t * UnmanagedWrapper::QueryUrl(const wchar_t * url)
	{
		String^ managedUrl = WCharToString(url, false);

		String^ managedResponse = _managedWrapper->QueryUrl(managedUrl);

		pin_ptr<const wchar_t> wch = PtrToStringChars(managedResponse);
		return const_cast<wchar_t*>(wch);
	}
}