/*
* The ScummVM WPF UI project
* Net Wrapper for WPF UI
*
* Copyright (c) 2016 MojoTouch
* Author: Ilya Rovensky
*/

#pragma once
#include "stdafx.h"

namespace NetWrapper {
	using namespace Enhanced;

	class ManagedWrapper;

	class ConvertedSetting
	{
	public:
		int Value;
		vector<bool> States;
		SettingControlType Type;
	};

	class ConvertedRadioSetting : public ConvertedSetting
	{
	public:
		vector<const wchar_t*>* ItemNames;
		vector<const wchar_t*>* ItemDescriptions;
	};

	class ConvertedSliderSetting : public ConvertedSetting
	{
	public:
		const wchar_t* Title;
		int MinValue;
		int MaxValue;
	};

	// Pure C++ data types container
	class __declspec(dllexport) UnmanagedWrapper
	{
	private:
		ManagedWrapper* _managedWrapper;
	public:
		UnmanagedWrapper();
		~UnmanagedWrapper();

		// Menu action ids
		static const int MenuActionSplash = 0;
		static const int MenuActionNewGame = 1;
		static const int MenuActionContinue = 2;
		static const int MenuActionSave = 3;
		static const int MenuActionLoad = 4;
		static const int MenuActionQuit = 5;
		static const int MenuActionSettings = 6;
		static const int MenuActionMain = 7;
		static const int MenuActionOpenHouse = 8;

		static const int MenuActionSettingsVoice = 9;
		static const int MenuActionSettingsMusic = 10;
		static const int MenuActionSettingsLanguage = 11;
		static const int MenuActionSettingsGraphics = 12;
		static const int MenuActionSettingsControls = 13;
		static const int MenuActionSettingsAbout = 14;
		static const int MenuActionSettingsBack = 15;

		static const int MenuActionTutorial = 16;
		static const int MenuActionProgress = 17;

		static const int MenuActionList = 18;
		static const int MenuActionListSlot1 = 19;
		static const int MenuActionListSlot2 = 20;
		static const int MenuActionListSlot3 = 21;
		static const int MenuActionListSlot4 = 22;
		static const int MenuActionListSlot5 = 23;
		static const int MenuActionListSlot6 = 24;
		static const int MenuActionListSlot7 = 25;
		static const int MenuActionListSlot8 = 26;

		static const int MenuActionSelectNegative = 27;
		static const int MenuActionSelectPositive = 28;
		static const int MenuActionInput = 29;
		static const int MenuActionAlert = 30;
		static const int MenuActionAlertButton1 = 31;
		static const int MenuActionAlertButton2 = 32;
		static const int MenuActionAlertButton3 = 33;

		static const int MenuActionDeveloperLogo1 = 41;
		static const int MenuActionDeveloperLogo2 = 42;

		static const int MenuActionExtras = 43;
		static const int MenuActionEnableOpenHouse = 44;

		// Shows menu with the specified menu action id
		// Returns the selected action
		int ShowMenu(int menu, int menuState);

		// Displays the menu without waiting for the user input
		void DisplayMainMenu(int menuState);

		// Show settings sub-menu
		int ShowSettings(int menu, vector<ConvertedSetting*>& settings);

		// Hides the current menu
		void HideMenu();

		// Displays a tutorial video
		//void ShowTutorial();

		// Shows a dialog with buttons
		int ShowDialog(const wchar_t* message, vector<const wchar_t*>& buttons);

		// Shows a dialog with a list
		int ShowDialogWithList(const wchar_t* message, vector<const wchar_t*>& items, int buttonCount, const wchar_t* negativeButton, const wchar_t* positiveButton);

		// Shows a dialog with an input textbox
		int ShowDialogWithInput(const wchar_t* message, wchar_t** input, const wchar_t* negativeButton, const wchar_t* positiveButton);

		// Shows a dialog with message and a spinner
		void ShowProgressDialog(const wchar_t* message);
		void HideProgressDialog();

		// Shows notifications and subtitles
		void ShowNotification(const wchar_t* message);

		void SetOglWindowHandle(HWND handle);

		void InitStrings(vector<const wchar_t*>& strings, int gameToPromote, const wchar_t* urlToPromote, const wchar_t* urlToRate);

		bool HasTouchScreen();

		bool IsShowingMenu();

		bool UpperCase;

		const wchar_t* QueryUrl(const wchar_t* url);
	};
}
